package uk.ac.bbsrc.babraham.SparkSpotter;
/*
 * SparkSpotter - An ImageJ plugin for finding bright sparks in movies
 * Copyright (c) 2007 Simon Andrews

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
import java.util.Vector;


public class SparkEventGroup {
	private int startFrame;
	private int endFrame;
	private double totalIntensity;
	private double totalFold;
	private int startX;
	private int endX;
	private int startY;
	private int endY;
	private Vector<SparkEvent> events;
	private int [] rawData;
	private int [] correctedData;
	
	public SparkEventGroup (SparkEvent event) {
		startFrame = event.startFrame();
		endFrame = event.endFrame();
		totalIntensity = event.averageIntensity();
		totalFold = event.averageFold();
		startX = event.getX();
		endX = event.getX();
		startY = event.getY();
		endY = event.getY();
				
		int [] r = event.rawData();
		rawData = new int [r.length];
		for (int i=0;i<r.length;i++) {
			rawData[i] = r[i];
		}

		int [] c = event.correctedData();
		correctedData = new int[c.length];
		for (int i=0;i<c.length;i++) {
			correctedData[i] = c[i];
		}

		events = new Vector<SparkEvent>();
		events.add(event);
		
	}
	
	public void addEvent (SparkEvent event) {
		if (event.endFrame() > endFrame) endFrame = event.endFrame();
		if (event.startFrame() < startFrame) startFrame = event.startFrame();
		
		if (event.getX() > endX) endX = event.getX();
		if (event.getX() < startX) startX = event.getX();
		if (event.getY() > endY) endY = event.getY();
		if (event.getY() < startY) startY = event.getY();
		
		totalIntensity += event.averageIntensity();
		totalFold += event.averageFold();
				
		int [] r = event.rawData();
		for (int i=0;i<rawData.length;i++) {
			rawData[i]+=r[i];
		}

		int [] c = event.correctedData();
		for (int i=0;i<correctedData.length;i++) {
			correctedData[i]+=c[i];
		}
		
		events.add(event);
	}
	
	public boolean overlaps (SparkEvent event, int pixels, int frames) {
		// Distance first
		if (event.getX()>=startX-pixels && event.getX()<=endX+pixels && event.getY()>=startY-pixels && event.getY()<=endY+pixels) {
			// It's a distance match.
			// Check the time next
			
			if (event.startFrame()>=startFrame-frames && event.endFrame()<=endFrame+frames) {
				// It all overlaps
				return true;
			}
			else {
				return false;
			}
		}
		else {
			return false;
		}
	}

	
	public boolean overlaps (SparkEventGroup event, int pixels, int frames) {
		// Distance first
		if (event.getEndX()>=startX-pixels && event.getStartX()<=endX+pixels && event.getEndY()>=startY-pixels && event.getStartY()<=endY+pixels) {
			// It's a distance match.
			// Check the time next
			
			if (event.startFrame()>=startFrame-frames && event.endFrame()<=endFrame+frames) {
				// It all overlaps
				return true;
			}
			else {
				return false;
			}
		}
		else {
			return false;
		}
	}

	public SparkEvent [] getEvents() {
		return (SparkEvent[])events.toArray(new SparkEvent[0]);	
	}
	
	public int startFrame () {
		return startFrame;
	}
	
	public int endFrame () {
		return endFrame;
	}
	
	public double averageIntensity () {
		return totalIntensity/events.size();
	}
	
	public double averageFold () {
		return totalFold/events.size();
	}
	
	public int [] rawData () {
		int [] r = new int[rawData.length ];
		for (int i=0;i<r.length;i++) {
			r[i] = rawData[i]/events.size();
		}
		return r;
	}

	public int [] correctedData () {
		int [] c = new int[correctedData.length ];
		for (int i=0;i<c.length;i++) {
			c[i] = correctedData[i]/events.size();
		}
		return c;
	}
	
	public int eventCount() {
		return events.size();
	}
	
	public int getStartX () {
		return startX;
	}
	public int getEndX () {
		return endX;
	}
	public int getStartY () {
		return startY;
	}
	public int getEndY () {
		return endY;
	}
	public int maxRaw () {
		int [] r = rawData();
		int max = r[1];
		for (int i=1;i<r.length;i++) {
			if (r[i]>max) max = r[i];
		}
		return max;
	}
	public int minRaw () {
		int [] r = rawData();
		int min = r[0];
		for (int i=1;i<r.length;i++) {
			if (r[i]<min) min = r[i];
		}
		return min;
	}
	public int maxCorrected () {
		int [] r = correctedData();
		int max = r[0];
		for (int i=1;i<r.length;i++) {
			if (r[i]>max) max = r[i];
		}
		return max;
	}
	public int minCorrected () {
		int [] r = correctedData();
		int min = r[0];
		for (int i=1;i<r.length;i++) {
			if (r[i]<min) min = r[i];
		}
		return min;
	}
	
}
